import json

from dataclasses import asdict
from typing import Any, Callable, Coroutine, List, Optional, Union

from aiohttp import web

from probirka import Probirka


def make_aiohttp_endpoint(
    probirka: Probirka,
    timeout: Optional[int] = None,
    with_groups: Union[str, List[str]] = '',
    skip_required: bool = False,
    return_results: bool = True,
    success_code: int = 200,
    error_code: int = 500,
) -> Callable[[web.Request], Coroutine[Any, Any, web.Response]]:
    """
    Create an aiohttp endpoint for a given Probirka instance.

    Args:
        probirka (Probirka): The Probirka instance to run.
        timeout (Optional[int]): The timeout for the Probirka run.
        with_groups (Union[str, List[str]]): Groups to include in the Probirka run.
        skip_required (bool): Whether to skip required checks.
        return_results (bool): Whether to return the results in the response.
        success_code (int): The HTTP status code for a successful response.
        error_code (int): The HTTP status code for an error response.

    Returns:
        Callable[[web.Request], Coroutine[Any, Any, web.Response]]: The aiohttp endpoint.
    """

    async def endpoint(
        _: web.Request,
    ) -> web.Response:
        """
        The aiohttp endpoint that runs the Probirka instance.

        Args:
            _: The aiohttp request object.

        Returns:
            web.Response: The HTTP response with the Probirka results.
        """
        res = await probirka.run(
            timeout=timeout,
            with_groups=with_groups,
            skip_required=skip_required,
        )
        status_code = success_code if res.ok else error_code
        return (
            web.json_response(
                text=json.dumps(obj=asdict(res), default=str),
                status=status_code,
            )
            if return_results
            else web.Response(
                body='',
                status=status_code,
            )
        )

    return endpoint



from abc import abstractmethod
from asyncio import iscoroutinefunction, wait_for
from datetime import datetime, timedelta
from typing import Callable, Optional, Union, Dict, Any, Protocol

from probirka._results import ProbeResult


class Probe(Protocol):
    """
    Protocol defining the interface for a probe.
    """

    @property
    def info(self) -> Optional[Dict[str, Any]]: ...

    def add_info(self, name: str, value: Any) -> None: ...

    async def run_check(self) -> ProbeResult: ...


class ProbeBase:
    """
    Base implementation of a probe.
    """

    def __init__(
        self,
        name: Optional[str] = None,
        timeout: Optional[int] = None,
        success_ttl: Optional[Union[int, timedelta]] = None,
        failed_ttl: Optional[Union[int, timedelta]] = None,
    ) -> None:
        """
        Initialize the probe.

        :param name: The name of the probe.
        :param timeout: The timeout for the probe.
        :param success_ttl: Cache duration for successful results. If None, successful results are not cached.
        :param failed_ttl: Cache duration for failed results. If None, failed results are not cached.
        """
        self._timeout = timeout
        self._name = name or self.__class__.__name__
        self._success_ttl = timedelta(seconds=success_ttl) if isinstance(success_ttl, int) else success_ttl
        self._failed_ttl = timedelta(seconds=failed_ttl) if isinstance(failed_ttl, int) else failed_ttl
        self._last_result: Optional[ProbeResult] = None
        self._info: Optional[Dict[str, Any]] = None

    def add_info(
        self,
        name: str,
        value: Any,
    ) -> None:
        """
        Add information to the probe result.

        :param name: The name of the information.
        :param value: The value of the information.
        """
        if self._info is None:
            self._info = {}
        self._info[name] = value

    @property
    def info(
        self,
    ) -> Optional[Dict[str, Any]]:
        """
        Get the information added to the probe result.

        :return: The information added to the probe result.
        """
        return self._info

    @abstractmethod
    async def _check(
        self,
    ) -> Optional[bool]:
        """
        Perform the check.

        :return: The result of the check.
        """
        raise NotImplementedError

    async def run_check(
        self,
    ) -> ProbeResult:
        """
        Run the check and return the result.

        :return: The result of the check.
        """
        now = datetime.now()
        use_cache = bool(self._success_ttl or self._failed_ttl)

        if self._last_result:
            last_check_time = self._last_result.started_at + self._last_result.elapsed
            ttl = self._success_ttl if self._last_result.ok else self._failed_ttl
            if ttl is not None:
                cache_until = last_check_time + ttl
                if now < cache_until:
                    return ProbeResult(
                        ok=self._last_result.ok,
                        started_at=self._last_result.started_at,
                        elapsed=self._last_result.elapsed,
                        name=self._last_result.name,
                        error=self._last_result.error,
                        info=self._last_result.info,
                        cached=bool(self._success_ttl is not None or self._failed_ttl is not None),
                    )

        started_at = now
        error = None
        task = self._check()
        try:
            result = await wait_for(
                fut=task,
                timeout=self._timeout,
            )
            if result is None:
                result = True
        except Exception as exc:
            result = False
            error = str(exc)

        probe_result = ProbeResult(
            ok=False if result is None else result,
            started_at=started_at,
            elapsed=datetime.now() - started_at,
            name=self._name,
            error=error,
            info=self._info,
            cached=False if use_cache else None,
        )

        if use_cache:
            ttl = self._success_ttl if probe_result.ok else self._failed_ttl
            if ttl is not None:
                self._last_result = probe_result

        return probe_result


class CallableProbe(ProbeBase):
    """
    A probe that wraps a callable function.
    """

    def __init__(
        self,
        func: Callable[[], Optional[bool]],
        name: Optional[str] = None,
        timeout: Optional[int] = None,
        success_ttl: Optional[Union[int, timedelta]] = None,
        failed_ttl: Optional[Union[int, timedelta]] = None,
    ) -> None:
        self._func = func
        super().__init__(
            name=name or func.__name__,
            timeout=timeout,
            success_ttl=success_ttl,
            failed_ttl=failed_ttl,
        )

    async def _check(
        self,
    ) -> Optional[bool]:
        """
        Perform the check by calling the function.

        :return: The result of the function call.
        """
        if iscoroutinefunction(self._func):
            return await self._func()
        return self._func()



import pytest

pytest_plugins = ('pytest_asyncio',)

from typing import Callable, Optional, Union
from unittest.mock import MagicMock

from pytest import fixture

from probirka import Probe, ProbeBase


@fixture
def make_testing_probe() -> Callable[[Union[MagicMock, Optional[bool]]], Probe]:
    def _inner(probe_result: Union[MagicMock, Optional[bool]]) -> Probe:
        class _Probe(ProbeBase):
            async def _check(self) -> Optional[bool]:
                if isinstance(probe_result, MagicMock):
                    return probe_result()
                return probe_result

        return _Probe()

    return _inner



from typing import Callable, Optional, Union, Any
from unittest.mock import MagicMock

import pytest
import asyncio
from datetime import datetime, timedelta
from unittest.mock import AsyncMock, patch

from probirka import Probe
from probirka._probes import ProbeBase, CallableProbe
from probirka._results import ProbeResult


@pytest.mark.parametrize(
    ['probe_result', 'is_ok'],
    [
        pytest.param(True, True),
        pytest.param(False, False),
        pytest.param(None, True),
        pytest.param(MagicMock(side_effect=ValueError('test error')), False),
    ],
)
@pytest.mark.asyncio
async def test_run_check(
    probe_result: Union[MagicMock, Optional[bool]],
    is_ok: bool,
    make_testing_probe: Callable[[Union[MagicMock, Optional[bool]]], Probe],
) -> None:
    probe = make_testing_probe(probe_result)
    results = await probe.run_check()
    assert results.ok == is_ok, results


class TestProbeBase:
    class ConcreteProbe(ProbeBase):
        async def _check(self) -> bool:
            return True

    def test_init_with_default_values(self) -> None:
        probe = self.ConcreteProbe()
        assert probe._name == "ConcreteProbe"
        assert probe._timeout is None

    def test_init_with_custom_values(self) -> None:
        probe = self.ConcreteProbe(name="CustomProbe", timeout=5)
        assert probe._name == "CustomProbe"
        assert probe._timeout == 5

    @pytest.mark.asyncio
    async def test_run_check_success(self) -> None:
        probe = self.ConcreteProbe()
        result = await probe.run_check()

        assert isinstance(result, ProbeResult)
        assert result.ok is True
        assert isinstance(result.started_at, datetime)
        assert isinstance(result.elapsed, timedelta)
        assert result.name == "ConcreteProbe"
        assert result.error is None

    @pytest.mark.asyncio
    async def test_run_check_with_timeout(self) -> None:
        class SlowProbe(ProbeBase):
            async def _check(self) -> bool:
                await asyncio.sleep(2)
                return True

        probe = SlowProbe(timeout=1)
        result = await probe.run_check()

        assert result.ok is False
        assert result.error is not None

    @pytest.mark.asyncio
    async def test_run_check_with_exception(self) -> None:
        class FailingProbe(ProbeBase):
            async def _check(self) -> bool:
                raise ValueError("Test error")

        probe = FailingProbe()
        result = await probe.run_check()

        assert result.ok is False
        assert result.error == "Test error"


class TestCallableProbe:
    def test_init_with_sync_function(self) -> None:
        def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        assert probe._name == "test_func"
        assert probe._func == test_func

    def test_init_with_async_function(self) -> None:
        async def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        assert probe._name == "test_func"
        assert probe._func == test_func

    @pytest.mark.asyncio
    async def test_check_with_sync_function(self) -> None:
        def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        result = await probe._check()
        assert result is True

    @pytest.mark.asyncio
    async def test_check_with_async_function(self) -> None:
        async def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        result = await probe._check()
        assert result is True

    @pytest.mark.asyncio
    async def test_run_check_with_sync_function(self) -> None:
        def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        result = await probe.run_check()

        assert isinstance(result, ProbeResult)
        assert result.ok is True
        assert result.name == "test_func"

    @pytest.mark.asyncio
    async def test_run_check_with_async_function(self) -> None:
        async def test_func() -> bool:
            return True

        probe = CallableProbe(test_func)
        result = await probe.run_check()

        assert isinstance(result, ProbeResult)
        assert result.ok is True
        assert result.name == "test_func"


@pytest.mark.asyncio
async def test_probe_caching() -> None:
    class TestProbe(ProbeBase):
        def __init__(self, success_ttl: Optional[int] = None, failed_ttl: Optional[int] = None):
            super().__init__(success_ttl=success_ttl, failed_ttl=failed_ttl)
            self._counter = 0

        async def _check(self) -> bool:
            self._counter += 1
            return True

    # Тест без кэширования
    probe = TestProbe()
    result1 = await probe.run_check()
    result2 = await probe.run_check()
    assert result1.ok is True
    assert result2.ok is True
    assert result1.cached is None
    assert result2.cached is None  # Когда кэширование отключено, cached всегда None
    assert probe._counter == 2

    # Тест с кэшированием успешного результата
    probe = TestProbe(success_ttl=1)
    result1 = await probe.run_check()
    await asyncio.sleep(0.1)  # Небольшая задержка, но меньше TTL
    result2 = await probe.run_check()
    assert result1.ok is True
    assert result2.ok is True
    assert result1.cached is False
    assert result2.cached is True
    assert probe._counter == 1

    # Проверка истечения срока действия кэша
    await asyncio.sleep(1.1)  # Ждем, пока истечет TTL
    result3 = await probe.run_check()
    assert result3.ok is True
    assert result3.cached is False
    assert probe._counter == 2

    # Тест с кэшированием неуспешного результата
    class FailingProbe(ProbeBase):
        def __init__(self, failed_ttl: Optional[int] = None):
            super().__init__(failed_ttl=failed_ttl)
            self._counter = 0

        async def _check(self) -> bool:
            self._counter += 1
            return False

    probe = FailingProbe(failed_ttl=1)
    result1 = await probe.run_check()
    await asyncio.sleep(0.1)  # Небольшая задержка, но меньше TTL
    result2 = await probe.run_check()
    assert result1.ok is False
    assert result2.ok is False
    assert result1.cached is False
    assert result2.cached is True
    assert probe._counter == 1

    # Проверка истечения срока действия кэша для неуспешного результата
    await asyncio.sleep(1.1)  # Ждем, пока истечет TTL
    result3 = await probe.run_check()
    assert result3.ok is False
    assert result3.cached is False
    assert probe._counter == 2


@pytest.mark.asyncio
async def test_probe_info() -> None:
    class TestProbe(ProbeBase):
        async def _check(self) -> bool:
            self.add_info("test_key", "test_value")
            return True

    probe = TestProbe()
    result = await probe.run_check()

    assert result.ok is True
    assert result.info == {"test_key": "test_value"}


@pytest.mark.asyncio
async def test_probe_info_caching() -> None:
    class TestProbe(ProbeBase):
        def __init__(self, success_ttl: Optional[int] = None):
            super().__init__(success_ttl=success_ttl)
            self._counter = 0

        async def _check(self) -> bool:
            self._counter += 1
            self.add_info("counter", self._counter)
            return True

    # Тест с кэшированием
    probe = TestProbe(success_ttl=1)
    result1 = await probe.run_check()
    result2 = await probe.run_check()

    assert result1.ok is True
    assert result2.ok is True
    assert result1.cached is False
    assert result2.cached is True
    assert result1.info == {"counter": 1}
    assert result2.info == {"counter": 1}
    assert probe._counter == 1


@pytest.mark.asyncio
async def test_probe_info_empty() -> None:
    class TestProbe(ProbeBase):
        async def _check(self) -> bool:
            return True

    probe = TestProbe()
    result = await probe.run_check()

    assert result.ok is True
    assert result.info == None



from typing import Any, Callable, Coroutine, List, Optional, Union

from fastapi import status
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse, Response

from probirka import Probirka


def make_fastapi_endpoint(
    probirka: Probirka,
    timeout: Optional[int] = None,
    with_groups: Union[str, List[str]] = '',
    skip_required: bool = False,
    return_results: bool = True,
    success_code: int = status.HTTP_200_OK,
    error_code: int = status.HTTP_500_INTERNAL_SERVER_ERROR,
) -> Callable[[], Coroutine[Any, Any, Response]]:
    """
    Create a FastAPI endpoint for a given Probirka instance.

    Args:
        probirka (Probirka): The Probirka instance to run.
        timeout (Optional[int]): The timeout for the Probirka run.
        with_groups (Union[str, List[str]]): Groups to include in the Probirka run.
        skip_required (bool): Whether to skip required checks.
        return_results (bool): Whether to return the results in the response.
        success_code (int): The HTTP status code for a successful response.
        error_code (int): The HTTP status code for an error response.

    Returns:
        Callable[[], Coroutine[Any, Any, Response]]: The FastAPI endpoint.
    """

    async def endpoint() -> Response:
        """
        The FastAPI endpoint that runs the Probirka instance.

        Returns:
            Response: The HTTP response with the Probirka results.
        """
        res = await probirka.run(
            timeout=timeout,
            with_groups=with_groups,
            skip_required=skip_required,
        )
        resp = JSONResponse(jsonable_encoder(res)) if return_results else Response()
        resp.status_code = success_code if res.ok else error_code
        return resp

    return endpoint



import pytest
from fastapi import FastAPI, status
from fastapi.testclient import TestClient

from probirka import Probirka, ProbeBase
from probirka._fastapi import make_fastapi_endpoint


class SuccessProbe(ProbeBase):
    async def _check(self) -> bool:
        return True


class FailureProbe(ProbeBase):
    async def _check(self) -> bool:
        return False


@pytest.fixture
def probirka() -> Probirka:
    return Probirka()


@pytest.fixture
def test_client(probirka: Probirka) -> TestClient:
    app = FastAPI()
    endpoint = make_fastapi_endpoint(probirka)
    app.add_api_route("/health", endpoint)
    return TestClient(app)


def test_successful_response(test_client: TestClient, probirka: Probirka) -> None:
    # Подготовка
    probirka.add_info("some_field", "value")
    probirka.add_probes(SuccessProbe())

    # Выполнение
    response = test_client.get("/health")

    # Проверка
    assert response.status_code == status.HTTP_200_OK
    response_data = response.json()
    assert response_data["ok"] is True
    assert response_data["info"]["some_field"] == "value"
    assert len(response_data["checks"]) == 1
    assert response_data["checks"][0]["ok"] is True


def test_error_response(test_client: TestClient, probirka: Probirka) -> None:
    # Подготовка
    probirka.add_probes(FailureProbe())

    # Выполнение
    response = test_client.get("/health")

    # Проверка
    assert response.status_code == status.HTTP_500_INTERNAL_SERVER_ERROR
    response_data = response.json()
    assert response_data["ok"] is False
    assert len(response_data["checks"]) == 1
    assert response_data["checks"][0]["ok"] is False


def test_custom_status_codes(probirka: Probirka) -> None:
    # Подготовка
    app = FastAPI()
    endpoint = make_fastapi_endpoint(
        probirka,
        success_code=status.HTTP_201_CREATED,
        error_code=status.HTTP_400_BAD_REQUEST
    )
    app.add_api_route("/health", endpoint)
    client = TestClient(app)

    probirka.add_probes(SuccessProbe())

    # Выполнение
    response = client.get("/health")

    # Проверка
    assert response.status_code == status.HTTP_201_CREATED
    response_data = response.json()
    assert response_data["ok"] is True


def test_without_results(probirka: Probirka) -> None:
    # Подготовка
    app = FastAPI()
    endpoint = make_fastapi_endpoint(
        probirka,
        return_results=False
    )
    app.add_api_route("/health", endpoint)
    client = TestClient(app)

    probirka.add_probes(SuccessProbe())

    # Выполнение
    response = client.get("/health")

    # Проверка
    assert response.status_code == status.HTTP_200_OK
    assert response.content == b""


def test_with_custom_parameters(probirka: Probirka) -> None:
    # Подготовка
    success_probe_1 = SuccessProbe()
    success_probe_2 = SuccessProbe()

    probirka.add_probes(success_probe_1)  # required probe
    probirka.add_probes(success_probe_2, groups=["group1"])  # optional probe

    app = FastAPI()
    endpoint = make_fastapi_endpoint(
        probirka,
        timeout=30,
        with_groups=["group1"],
        skip_required=True
    )
    app.add_api_route("/health", endpoint)
    client = TestClient(app)

    # Выполнение
    response = client.get("/health")

    # Проверка
    assert response.status_code == status.HTTP_200_OK
    response_data = response.json()
    assert response_data["ok"] is True
    # Проверяем, что запустился только один проб из группы group1
    assert len(response_data["checks"]) == 1



from contextlib import suppress

from probirka._probes import Probe, ProbeBase
from probirka._probirka import Probirka
from probirka._results import ProbirkaResult, ProbeResult

__title__ = 'probirka'
__version__ = '0.4.1'
__url__ = 'https://github.com/appKODE/probirka'
__author__ = 'KODE'
__author_email__ = 'slurm@kode.ru'
__license__ = 'MIT'
__description__ = 'A health check library for Python applications'
__all__ = [
    'Probe',
    'ProbeBase',
    'ProbeResult',
    'Probirka',
    'ProbirkaResult',
]

with suppress(ImportError):
    from probirka._aiohttp import make_aiohttp_endpoint  # noqa

    __all__.append('make_aiohttp_endpoint')

with suppress(ImportError):
    from probirka._fastapi import make_fastapi_endpoint  # noqa

    __all__.append('make_fastapi_endpoint')



from asyncio import gather, wait_for
from collections import defaultdict
from datetime import datetime, timedelta
from typing import Any, Callable, Dict, List, Optional, Sequence, Union

from probirka._probes import CallableProbe, Probe
from probirka._results import ProbirkaResult, ProbeResult


class Probirka:
    """
    Probirka is a health check manager that allows adding and running probes.
    """

    def __init__(
        self,
        success_ttl: Optional[Union[int, timedelta]] = None,
        failed_ttl: Optional[Union[int, timedelta]] = None,
    ) -> None:
        """
        Initialize the Probirka instance.

        :param success_ttl: Default cache duration for successful results. If None, successful results are not cached.
        :param failed_ttl: Default cache duration for failed results. If None, failed results are not cached.
        """
        self._required_probes: List[Probe] = []
        self._optional_probes: Dict[str, List[Probe]] = defaultdict(list)
        self._info: Optional[Dict[str, Any]] = None
        self._success_ttl = timedelta(seconds=success_ttl) if isinstance(success_ttl, int) else success_ttl
        self._failed_ttl = timedelta(seconds=failed_ttl) if isinstance(failed_ttl, int) else failed_ttl

    def add_info(
        self,
        name: str,
        value: Any,
    ) -> None:
        """
        Add information to the health check result.

        :param name: The name of the information.
        :param value: The value of the information.
        """
        if self._info is None:
            self._info = {}
        self._info[name] = value

    @property
    def info(
        self,
    ) -> Optional[Dict[str, Any]]:
        """
        Get the information added to the health check result.

        :return: The information added to the health check result.
        """
        return self._info

    def add_probes(
        self,
        *probes: Probe,
        groups: Union[str, List[str]] = '',
    ) -> None:
        """
        Add probes to the health check.

        :param probes: Probes to add
        :param groups: Groups for optional probes. Probes without groups are required.
        """
        if groups:
            if isinstance(groups, str):
                groups = [groups]
            for group in groups:
                self._optional_probes[group].extend(probes)
            return
        self._required_probes.extend(probes)

    def add(
        self,
        name: Optional[str] = None,
        timeout: Optional[int] = None,
        groups: Union[str, List[str]] = '',
        success_ttl: Optional[Union[int, timedelta]] = None,
        failed_ttl: Optional[Union[int, timedelta]] = None,
    ) -> Callable:
        """
        Decorator to add a callable as a probe.

        :param name: Probe name
        :param timeout: Probe timeout in seconds
        :param groups: Groups for optional probes. Probes without groups are required.
        :param success_ttl: Cache duration for successful results. If None, uses the global success_ttl setting.
        :param failed_ttl: Cache duration for failed results. If None, uses the global failed_ttl setting.
        :return: Decorated function
        """

        def _wrapper(func: Callable) -> Any:
            self.add_probes(
                CallableProbe(
                    func=func,
                    name=name,
                    timeout=timeout,
                    success_ttl=success_ttl or self._success_ttl,
                    failed_ttl=failed_ttl or self._failed_ttl,
                ),
                groups=groups,
            )
            return func

        return _wrapper

    async def _inner_run(
        self,
        with_groups: List[str],
        skip_required: bool,
    ) -> Sequence[ProbeResult]:
        """
        Run probes and gather results.

        :param with_groups: Groups to run. Required probes run unless skip_required=True
        :param skip_required: Skip probes without groups
        :return: Sequence of probe results
        """
        tasks = [] if skip_required else [probe.run_check() for probe in self._required_probes]
        for group in with_groups:
            tasks += [probe.run_check() for probe in self._optional_probes[group]]
        results = await gather(*tasks)
        for coro in tasks:
            coro.close()
        return results

    async def run(
        self,
        timeout: Optional[int] = None,
        with_groups: Union[str, List[str]] = '',
        skip_required: bool = False,
    ) -> ProbirkaResult:
        """
        Run health check and return results.

        :param timeout: Overall timeout in seconds
        :param with_groups: Groups to run. Required probes run unless skip_required=True
        :param skip_required: Skip probes without groups
        :return: Health check result
        """
        if with_groups and isinstance(with_groups, str):
            with_groups = [with_groups]
        started_at = datetime.now()
        fut = self._inner_run(
            with_groups=with_groups,  # type: ignore
            skip_required=skip_required,
        )
        results = (
            await wait_for(
                fut=fut,
                timeout=timeout,
            )
            if timeout
            else await fut
        )
        ok = True
        for result in results:
            if result.ok is False:
                ok = False
                break
        return ProbirkaResult(
            ok=ok,
            info=self._info,
            started_at=started_at,
            elapsed=datetime.now() - started_at,
            checks=results,
        )



import pytest
from probirka import Probirka


@pytest.mark.asyncio
async def test_decorator() -> None:
    checks = Probirka()
    results = await checks.run()
    assert not results.checks

    @checks.add()
    def ok_check() -> bool:
        return False

    results = await checks.run()
    assert results.checks


@pytest.mark.asyncio
async def test_optional_probe() -> None:
    checks = Probirka()

    @checks.add()
    def _check_1() -> bool:
        return True

    @checks.add(groups='optional')
    def _check_2() -> bool:
        return False

    # Проверяем, что при запуске только опциональных проверок запускается только одна проверка
    results = await checks.run(with_groups='optional', skip_required=True)
    assert len(results.checks) == 1
    assert results.checks[0].ok is False

    # Проверяем, что при запуске всех проверок запускается только обязательная проверка
    results = await checks.run()
    assert len(results.checks) == 1
    assert results.checks[0].ok is True


@pytest.mark.asyncio
async def test_probirka_caching() -> None:
    checks = Probirka()
    counter = 0

    @checks.add(success_ttl=1)
    def _check_1() -> bool:
        nonlocal counter
        counter += 1
        return True

    # Первый запуск
    results = await checks.run()
    assert results.checks[0].ok is True
    assert counter == 1

    # Второй запуск (должен использовать кэш)
    results = await checks.run()
    assert results.checks[0].ok is True
    assert counter == 1

    # Проверка глобальных настроек кэширования
    checks2 = Probirka(success_ttl=1, failed_ttl=1)
    counter2 = 0

    @checks2.add()
    def _check_2() -> bool:
        nonlocal counter2
        counter2 += 1
        return True

    # Первый запуск
    results = await checks2.run()
    assert results.checks[0].ok is True
    assert counter2 == 1

    # Второй запуск (должен использовать кэш)
    results = await checks2.run()
    assert results.checks[0].ok is True
    assert counter2 == 1



from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Any, Dict, Optional, Sequence


@dataclass(frozen=True, order=True)
class ProbeResult:
    name: str
    ok: bool
    cached: Optional[bool]
    started_at: datetime
    elapsed: timedelta
    info: Optional[Dict[str, Any]]
    error: Optional[str]


@dataclass(frozen=True, order=True)
class ProbirkaResult:
    ok: bool
    started_at: datetime
    elapsed: timedelta
    info: Optional[Dict[str, Any]]
    checks: Sequence[ProbeResult]



import json
from datetime import datetime, timedelta
from typing import AsyncGenerator

import pytest
from aiohttp import web
from aiohttp.test_utils import TestClient, TestServer

from probirka import Probirka, ProbeBase
from probirka._aiohttp import make_aiohttp_endpoint
from probirka._results import ProbirkaResult, ProbeResult


class SuccessProbe(ProbeBase):
    async def _check(self) -> bool:
        return True


class FailureProbe(ProbeBase):
    async def _check(self) -> bool:
        return False


@pytest.fixture
def probirka() -> Probirka:
    return Probirka()


@pytest.mark.asyncio
async def test_successful_response(probirka: Probirka) -> None:
    # Подготовка
    app = web.Application()
    endpoint = make_aiohttp_endpoint(probirka)
    app.router.add_get("/health", endpoint)
    server = TestServer(app)

    probirka.add_info("some_field", "value")
    probirka.add_probes(SuccessProbe())

    async with TestClient(server) as client:
        # Выполнение
        async with client.get("/health") as response:
            # Проверка
            assert response.status == 200
            response_data = await response.json()
            assert response_data["ok"] is True
            assert response_data["info"]["some_field"] == "value"
            assert len(response_data["checks"]) == 1
            assert response_data["checks"][0]["ok"] is True


@pytest.mark.asyncio
async def test_error_response(probirka: Probirka) -> None:
    # Подготовка
    app = web.Application()
    endpoint = make_aiohttp_endpoint(probirka)
    app.router.add_get("/health", endpoint)
    server = TestServer(app)

    probirka.add_probes(FailureProbe())

    async with TestClient(server) as client:
        # Выполнение
        async with client.get("/health") as response:
            # Проверка
            assert response.status == 500
            response_data = await response.json()
            assert response_data["ok"] is False
            assert len(response_data["checks"]) == 1
            assert response_data["checks"][0]["ok"] is False


@pytest.mark.asyncio
async def test_custom_status_codes(probirka: Probirka) -> None:
    # Подготовка
    app = web.Application()
    endpoint = make_aiohttp_endpoint(
        probirka,
        success_code=201,
        error_code=400
    )
    app.router.add_get("/health", endpoint)
    server = TestServer(app)

    probirka.add_probes(SuccessProbe())

    async with TestClient(server) as client:
        # Выполнение
        async with client.get("/health") as response:
            # Проверка
            assert response.status == 201
            response_data = await response.json()
            assert response_data["ok"] is True


@pytest.mark.asyncio
async def test_without_results(probirka: Probirka) -> None:
    # Подготовка
    app = web.Application()
    endpoint = make_aiohttp_endpoint(
        probirka,
        return_results=False
    )
    app.router.add_get("/health", endpoint)
    server = TestServer(app)

    probirka.add_probes(SuccessProbe())

    async with TestClient(server) as client:
        # Выполнение
        async with client.get("/health") as response:
            # Проверка
            assert response.status == 200
            assert await response.text() == ""


@pytest.mark.asyncio
async def test_with_custom_parameters(probirka: Probirka) -> None:
    # Подготовка
    success_probe_1 = SuccessProbe()
    success_probe_2 = SuccessProbe()

    probirka.add_probes(success_probe_1)  # required probe
    probirka.add_probes(success_probe_2, groups=["group1"])  # optional probe

    app = web.Application()
    endpoint = make_aiohttp_endpoint(
        probirka,
        timeout=30,
        with_groups=["group1"],
        skip_required=True
    )
    app.router.add_get("/health", endpoint)
    server = TestServer(app)

    async with TestClient(server) as client:
        # Выполнение
        async with client.get("/health") as response:
            # Проверка
            assert response.status == 200
            response_data = await response.json()
            assert response_data["ok"] is True
            # Проверяем, что запустился только один проб из группы group1
            assert len(response_data["checks"]) == 1
